#*****************************************************
# RPM Wizard
# Copyright 2001,2002 Magnus Bjorklof <bjorklof@nic.fi>
# License: GNU General Public License
#*****************************************************


import sys
import os
import string
import re
import det
import rpm
import tempfile
import shutil
import databasereader

from rpmwizdbcreator import DBCreator


from test import *
from conf import *
from shortcut import *

from pageOne import PageOne
from pageTwo import PageTwo
from pageThree import PageThree

from qt import *





class RpmWiz(QWizard):
    def __init__(self,parent = None,name = None,modal = 0,fl = 0):
        QWizard.__init__(self,parent,name,modal,fl)

	icon=QPixmap(APPDIR + './rpmwiz.png')

	self.setIcon(icon)
        self.setCaption(NAME)

        self.resize(570,413)
	self.setMinimumSize(570,413)
        self.setMaximumSize(700,420)

	self.currentp=1

	self.pageOne = PageOne(self,'page', APPDIR)
	self.addPage(self.pageOne,self.tr('Select file'))

        self.pageTwo = PageTwo(self,'page2', APPDIR)
	self.addPage(self.pageTwo,'Information')

        self.pageThree = PageThree(self,'page3', APPDIR)
	self.addPage(self.pageThree,self.tr(''))

	self.connect(self.pageOne.PushButton1,SIGNAL('clicked()'),self.getFileName)
        self.connect(self.pageOne.LineEdit1,SIGNAL('textChanged(const QString&)'),self.enableNext)
	self.connect(self.pageTwo.PushButtonV,SIGNAL('clicked()'),self.details)

        self.setNextEnabled(self.pageOne, 0)
        self.pageOne.PushButton1.setDefault(1)
        self.helpButton().setText('About...')

	self.cont=''
	self.resizeCount=0

	self.dep_satisfiers=[]
	self.tempfiles=[]

	self.databases='' #these are the databases for the local files, not the premade databases
	
    def getHeader(self, filename):
	"""Get the RPM header from the file and return a tuple (header, isSource)"""
	fd = os.open(filename, os.O_RDONLY)
        h=rpm.headerFromPackage(fd)
	os.close(fd)
	return h

    def details(self):
    	"""This method is called when the user clicks the 'Details...' button."""
	qApp.setOverrideCursor(Qt.waitCursor)

	self.getDetails=det.GetDetails(self.header)
	(dets, files)=self.getDetails.get()
	self.dt=det.DetailsDialog(self)
 	self.dt.setValues(dets,files)
        self.dt.setCaption('Details - '+NAME)

	qApp.restoreOverrideCursor()
        #self.dt.show()
	self.dt.exec_loop()
    def enableNext(self):
    	"""Enables the 'Next' button on first page. Only done if the filename is set."""
    	if self.fileName()!='':

		self.setNextEnabled(self.pageOne, 1)
		self.nextButton().setDefault(1)

	else:
        	pass

    def fileName(self, c=0):
    	"""Method that represents the filename."""
        if c==0:
    		return str(self.pageOne.LineEdit1.text())
	elif c==1:
        	s=string.strip(str(self.pageOne.LineEdit1.text()))
		ss=string.replace(s, ' ', '\ ')
		return ss
	else:
        	pass

    def getFileName(self):
    	"""Open a file dialog to get the name of the rpm file to install."""

	# this code does not depend on the file existing; only the directory
	if os.path.exists(self.fileName()+'/')==1: # in this case self.fileName() is a directory
		path=self.fileName()
	elif os.path.exists(os.path.dirname(self.fileName()))==1:
		path=os.path.dirname(self.fileName())
	elif self.fileName()[:4]=='/mnt':
		path='/mnt/'
	else:
		path='/'


	fn = QFileDialog()
	fname = fn.getOpenFileName(path , 'RPM Files (*.rpm);;All Files (*)', self, 'Open file', 'Open...')

        if str(fname)!='':
        	self.pageOne.LineEdit1.setText(fname)
	else:
        	pass


    def getInfo(self):
        """Get some information from the RPM file and manipulate it to look better."""

 	desc3=self.header[rpm.RPMTAG_DESCRIPTION]
	desc2=re.split('\012\012', desc3)
	stri=''

        for x in desc2:
		s=string.replace(x, '\012', ' ')
                stri=stri + s + '\012\012'

        self.desc=string.strip(stri)
	self.pageTwo.TextBrowser.setText(self.desc)

	self.vers = self.header[rpm.RPMTAG_VERSION]
        self.name2 = self.header[rpm.RPMTAG_NAME]
	self.name = self.name2 + ' ' + self.vers
	self.pageTwo.TextLabel1_2.setText(self.name)


    def resizeEvent(self, ev):
    	"""Handles some resizing (this code sucks :( I should replace it with proper layouts)."""

        self.resizeCount=self.resizeCount+1
        if self.resizeCount>1:
                #cwidth is the change in width of the window
                cwidth=(self.width() - ev.oldSize().width())
		self.pageTwo.LayoutWidget.resize((self.pageTwo.LayoutWidget.width() + cwidth), self.pageTwo.LayoutWidget.height())


    def installRpm(self):
    	"""Handles the install process."""

        qApp.processEvents()
	self.testRpm()
	qApp.processEvents()

        if self.cont == 1:
                arg=0
	elif self.cont == 2:
                arg=255
        elif self.cont == 0:
		return "restart"


	db = rpm.opendb(1, '/')
	ts = rpm.TransactionSet('/', db)

	# if we only have to install one file, that file is self.fileName()
	# otherwise all files can be found in self.dep_satisfiers
	if self.dep_satisfiers==[]:
		ts.add(self.header,(self.header[rpm.RPMTAG_NAME], self.fileName()), 'u')

	else:
		alreadyAdded=[]
		print "Files to install:", self.dep_satisfiers
		for (hdr, file) in self.dep_satisfiers:
			for name in alreadyAdded:
				if name==hdr[rpm.RPMTAG_NAME]:
					break
			else:
				ts.add(hdr,(hdr[rpm.RPMTAG_NAME], file), 'u')
				alreadyAdded.append(hdr[rpm.RPMTAG_NAME])

	ts.order()

	self.installOK=0
	
	self.takeSomeTime()

	run_output = ts.run(0, arg, self.callback, "olle") 
	
	qApp.processEvents()
	if self.installOK==0:
		self.pageThree.TextLabel3.setText("<b>An error occured while installing!</b><br>The installation did not finish.")
	else:
		self.pageThree.TextLabel3.setText(self.tr('<b>Congratulations! Installation successful.</b><br>Enjoy your new software!'))
	if run_output==None:
		print "Installation of "+self.name+" successful"
	else:
		print "Error occured while installing:", run_output

	del db
	del ts
	
	qApp.processEvents()

	for tmpfile in self.tempfiles:
		print commands.getoutput('rm -fv '+tmpfile)


	self.scProgName=self.sc.programName()
	if self.scProgName!=None:
		self.pageThree.launchCheckbox1.show()
		self.pageThree.launchCheckbox1.setText('Launch '+self.scProgName)
	else:
		self.pageThree.launchCheckbox1.hide()
	
        self.pageThree.changePage('c')
        self.setFinishEnabled(self.pageThree, 1)
	self.setHelpEnabled(self.pageThree, 1)
	self.finishButton().setDefault(1)
        self.helpButton().setEnabled(1)

	return None

    def help(self):
    	"""Display the help dialog."""
        self.about=QMessageBox.about(self, 'About RPM Wizard', """<b>"""+ NAME+' '+APPVERSION+"""</b>
<br><br>Copyright (c) 2001, 2002 """+ """Magnus Bjrklf &#60;bjorklof@nic.fi&#62;<br><br>
Web site: """+HOMEPAGE+"""<br><br>
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.""")


    def next(self):
        """This is what happens when the user clicks the Next button. self.currentp is the
	current page when the user pushes the button."""

	if self.currentp==1:
		qApp.setOverrideCursor(Qt.waitCursor)
		self.setPage2(firstPage=0)
		qApp.restoreOverrideCursor()
		
	elif self.currentp==2:
        	self.currentp=3

		self.pageThree.changePage('a')
                self.showPage(self.pageThree)
        	self.cancelButton().setEnabled(0)
        	self.helpButton().setEnabled(0)
		self.finishButton().setEnabled(0)
		self.setBackEnabled(self.pageThree, 0)
		self.setHelpEnabled(self.pageThree, 0)
		qApp.processEvents()
		self.sc=Shortcut()
		self.desktopFiles=self.sc.files(self.header)
		if self.desktopFiles==[]:
			self.pageThree.desktopCheckbox.hide()
		else:
			cnt=len(self.desktopFiles)
			self.pageThree.desktopCheckbox.setText('Create shortcuts ('+str(cnt)+' found) on desktops')

        	self.takeSomeTime(2000)
		
		try:
			r=self.installRpm()
		except rpm.error:
			print "rpm.error"
			text="An unexpected error occurred."
			mb = QMessageBox(NAME, text, QMessageBox.Information, QMessageBox.Yes | QMessageBox.Default , QMessageBox.Cancel, QMessageBox.NoButton, self)
			mb.setButtonText(QMessageBox.Yes, "Restart")
			mb.setButtonText(QMessageBox.Cancel, "Exit")
			mb.exec_loop()
			if mb.result() == QMessageBox.Yes:
				self.restart()
			else:
				sys.exit(0)


		if r=="restart":
			self.restart()

	else:
        	pass


    def back(self):
    	"""This method is called when the user clicks the 'Back' button.
	Can only be used from the second page"""

	self.showPage(self.pageOne)
	self.currentp=1
	self.setCaption(NAME)


    def restart(self):
    	"""Restart the wizard."""

        self.currentp=1
	self.setCaption(NAME)

	self.qfii=QFileInfo(self.pageOne.LineEdit1.text())
	self.pageOne.LineEdit1.setText(self.qfii.dirPath(1))

	self.pageThree.CheckBox1.setChecked(0)
	self.pageThree.desktopCheckbox.show()
	self.pageThree.desktopCheckbox.setChecked(0)
	self.pageThree.launchCheckbox1.setChecked(0)

	self.dep_satisfiers=[]
	self.pageThree.changePage('a')
	self.pageOne.PushButton1.setDefault(1)
        self.cancelButton().setEnabled(1)
	self.setNextEnabled(self.pageOne, 0)
	self.setBackEnabled(self.pageTwo, 1) # new
	self.cont=''

	self.showPage(self.pageOne)

    def accept(self):
 	"""This is run when user clicks 'Finish'. If the user has chosen to install more packages
	the wizard will restart."""
	if self.pageThree.desktopCheckbox.isChecked()==1:
		qApp.setOverrideCursor(Qt.waitCursor)
		self.sc.copy()
		qApp.restoreOverrideCursor()
	else:
		pass

	if self.pageThree.launchCheckbox1.isChecked()==1:
		self.sc.launchProgram(self.scProgName)
	else:
		pass

	if self.pageThree.CheckBox1.isChecked()==1:
        	self.restart()
	else:
 		sys.exit(0)

    def takeSomeTime(self, time=1000):
        """This is just an ugly hack, needs work
        Needed to process some events so that the GUI will update"""

	x=0
	while x<time:
                x=x+1
                qApp.processEvents()

    def testRpm(self):
        """Test the RPM file on page 3 before installing."""

	if isKernel(self.name2, self.isSource)==1:
		testStatus='kernel'
		text2="""<b>WARNING!</b> It looks like you are about to install a kernel.
Don't use RPM Wizard to do that. Install it from the command line with 'rpm -i kernel.rpm' instead."""
		self.warningMsg(text2)
	else:
		pass

	if self.cont==0:
		return
	else:
		pass



	self.pageThree.page3a.setText(self.tr('Testing package...'))
	qApp.processEvents()
	
	self.runTest()
	qApp.processEvents()

	if self.cont==0:
		return
	else:
		pass


	dir=os.path.dirname(self.fileName())
	if fnmatch.fnmatch(dir, '*cdrom*')!=1:
		# it's a bit unneccessery that this is done if there are no missing
		# dependencies, maybe change that in the future
		self.pageThree.page3a.setText(self.tr('Creating database...'))
		qApp.processEvents()
    		files=commands.getoutput('ls '+dir+'/*.rpm')
    		filelist=string.split(files)
    		dbcr=DBCreator()
    		self.databases=dbcr.listCreator(filelist)



	self.pageThree.page3a.setText(self.tr('Checking dependencies...'))
	qApp.processEvents()

	missing_deps, conflicts, ip_missing_deps, dep_sat=depCheck(self, self.header, self.databases, details_dialog=0)
	
	qApp.processEvents()


	if dep_sat!=[]:
		self.pageThree.page3a.setText(self.tr('Package is missing dependencies. Searching...')) 

		qApp.processEvents()

		dep_sat=self.removeDuplicate(dep_sat)
		(dep_sat, stillUnsatisfied)=self.checkDepRecursive(dep_sat)

		for dep in stillUnsatisfied:
			if missing_deps=='' or missing_deps==None:
				missing_deps=dep
			else:
				missing_deps=missing_deps+', '+dep



	if conflicts!=None:
		text2="Package <b>"+conflicts+"""</b>, which is installed on your system, conflicts with
the package you are trying to install."""
		self.warningMsg(text2)
	else:
		pass

	if self.cont==0:
		return
	else:
		pass
	

	if ip_missing_deps!=None:
		text2="""<b></b>This installation will replace an existing package. The following packages, which are installed on your system, still need the old
version and may not work anymore if you continue.<br><br><b>"""+ ip_missing_deps+"</b>"
		self.warningMsg(text2)
	else:
		pass

	if self.cont==0:
		return
	else:
		pass
	
	if missing_deps!=None and missing_deps!='':
		testStatus=missing_deps
		text2=self.name + " needs the following to work correctly:<br><br><b>" + testStatus + """</b>
<br><br>You should probably install packages containing these before continuing."""
		self.warningMsg(text2)
	else:
		pass

	if self.cont==0:
		return
	else:
		pass



	if dep_sat!=None and dep_sat!=[]:

	    self.pageThree.page3a.setText(self.tr('Sorting the list of files...'))
	    qApp.processEvents()

	    dep_sat=self.removeDuplicate(dep_sat)
	    qApp.processEvents()
	    self.tempFile(self.fileName())

	    for ((headr,file), dep, storagename, type) in dep_sat:
	    	if type=='file':
			h7, prrrr=self.getHeader(file)
			self.dep_satisfiers.append((h7, file))
			dep_sat.remove(((headr,file), dep, storagename, type))
		elif type[:9]=="removable":
			pass
		else:
			print "Oh no, we don't support this yet..."
			dep_sat.remove(((headr,file), dep, storagename, type))

	    # only removables should pass this line

	    storagenameslist=self.sortStoragenames(dep_sat)

	    self.pageThree.page3a.setText(self.tr('Adding files...'))
	    qApp.processEvents()

	    self.handleStoragenameslist(storagenameslist)
	    qApp.processEvents()


	

    def checkDepRecursive(self, dep_sat):
	"""Check if the dep files have any missing dependencies themselves and return a list
	of files that need to be installed and a list of unsatisfied dependencies."""


	d_sat=dep_sat
	rpmdb = rpm.opendb()

	dependslist=databasereader.getDepDatabase(self.databases)


	alreadyChecked=[]

	while 1: #we restart the loop until allUnsatisfied is empty
	    allUnsatisfied=[]
	    stillUnsatisfied=[]
	    

	    for f in d_sat:

		file=f[0][1]
		
		dependencies=[]

		for row in dependslist:
			row=string.split(row, '@')

	    		if fnmatch.fnmatch(file, '*'+row[0]+'*')==1: # row[0] is the short filename
				if row[0]!='':
					dependencies=row[1:]
					break


		unsat=[]
		for dep0 in dependencies:
			foundIt=0
			for d in alreadyChecked:
				if d==dep0:
					foundIt=1
					break
			if foundIt==1:
				continue

			alreadyChecked.append(dep0)

			if dep0[:6]=='rpmlib':
				continue


			dep=string.split(dep0)

        		indexes = rpmdb.findbyprovides(dep[0])

			if len(indexes)==0:
				indexes2 = rpmdb.findbyfile(dep[0])
				if len(indexes2)==0:

					unsat.append(dep0)
				continue

			# now we check if we have found the right version
			foundIt=0
			for index in indexes:
				hdr=rpmdb[index]
				provnames=hdr[rpm.RPMTAG_PROVIDENAME]
				provversions=hdr[rpm.RPMTAG_PROVIDEVERSION]
				provflags=hdr[rpm.RPMTAG_PROVIDEFLAGS]
				
				try: # provflags is not a list if it is only one item, this fixes it
					provflags[0]
				except:
					provflags=[provflags]

				t=0
				for p in provnames:

				    pstring=makeRequireString(p, provversions[t], provflags[t])
				    if databasereader.compareStrings(dep0, pstring)==1:
				    	foundIt=1
					break
				    t=t+1
				if foundIt==1:
					break
			else:
				unsat.append(dep0)



		allUnsatisfied=allUnsatisfied+unsat
	    

	    if allUnsatisfied!=[]:

	    	d23=databasereader.searchDepSatisfier(allUnsatisfied, self.databases)

		if len(allUnsatisfied)==len(d23):
			pass
		else:
			# we have to compare the lists and check which were not satisfied

			for item in allUnsatisfied:
				for d_item in d23:
					if item==d_item[1]:
						break
				else:
					stillUnsatisfied.append(item)

	    	dep_sat=dep_sat+d23

		d_sat=d23 # we run the loop again, this time with new items in the list

		
	    else:
	    	break


	del rpmdb

	return (dep_sat, stillUnsatisfied)

    def runTest(self):
    	"""Do a test install and get the output."""
	db = rpm.opendb(1, '/') #we could maybe remove the arguments (check in doc)
	ts = rpm.TransactionSet('/', db)
	ts.add(self.header, self.header, 'u')

	test = ts.run(rpm.RPMTRANS_FLAG_BUILD_PROBS, 0, self.testCallback, 0)
	del db
	del ts

	if test!=[] and test!=None:
	    if len(test)==1:
		(errorText, other)=test[0]
		
		text2="This problem occured while trying to install " + self.name + ":<br><br><b>" + errorText + """</b>"""
		self.warningMsg(text2)
	    elif len(test)>1:
	    	errorText=''
		g=0
	    	for (err, (da, dada, dadada)) in test:
			g=g+1
			print err
			if errorText=='':
				errorText=err
			else:
			    if g<4:
				errorText=errorText+'<br><br>'+err
			    else:
			        # avoid putting too much text in the error dialog
				# by only displaying the first 3. The other are only
				# printed in the console
			    	pass

		text2="These problems occured while trying to install " + self.name + ":<br><br><b>" + errorText + """</b>"""

		self.warningMsg(text2)
	else:
		if self.cont==2:
			pass
		else:
			self.cont = 1

    def removeDuplicate(self,dep_sat):
	    """Remove some entries in dep_sat list if there are more than one item of each"""

	    list=[]
	    for i in dep_sat:

		for i2 in list:
		    if i[0]==i2[0]:
			break
	    	else:
	    		list.append(i)

	    return list


    def sortStoragenames(self,dep_sat):
	    """Sort the storagenames, add the files to the correct storagenames and return the sorted list"""

	    # we group the items in dep_sat by storagename so that you don't have to put the same disk
	    # in the drive twice


	    # do a list of all storagenames
	    storagenameslist=[]
	    for ((headr,file), dep, storagename, type) in dep_sat:
		try:
			storagenameslist.index([storagename])
		except:
			storagenameslist.append([storagename])
	    
	    
	    if len(storagenameslist)>1:
			
		rpmdatabaselist=[]
		

		for stlist in storagenameslist:
			st=stlist[0]
			if st!='' and st!=None:
				rpmdatabaselist.append('list.'+st)

		for dbfile in rpmdatabaselist:
		 	try:
				fd=open(DATABASEDIR+'/'+dbfile)
				rpmfileslist=fd.readlines()
				fd.close()
			except:
				print "Could not open", dbfile
				continue
			for row in rpmfileslist:
				r=string.split(row, ':/')

				if len(r)<2:
					continue
				else:
					pass

			
				if fnmatch.fnmatch(r[1], self.fileName())==1:
					# the storagename that has the file we are installing
					# is put first in the list so that we start taking the files
					# from that media
					storagenameslist.remove([dbfile[5:]])
					storagenameslist.insert(0, [dbfile[5:]])
					break 
	    else:
	    		pass

	    for ((headr,file), dep, storagename, type) in dep_sat:
	    	for li23 in storagenameslist:
			if storagename==li23[0]:
				li23.append(((headr,file), dep, storagename, type))
	    
	    return storagenameslist


    def handleStoragenameslist(self, storagenameslist):
	    """Add the files that are needed for install and ask the user to insert the media if necessery."""
           
	    notFoundDeps=[] #there are dependencies that could not be found even though they were in the list
	    # most often means that the user does not have that particular CD

	    for dep_satList in storagenameslist:
		for kim in dep_satList:
			try:
	    			((headr,file), dep, storagename, type)=kim
			except: #now we probably have the first item of the list which is the name of the media and not needed
				continue
			if type[:9]=="removable":

				pass
			elif type=='file':
				h7, prrrr=self.getHeader(file)
				self.dep_satisfiers.append((h7, file))
				continue
			else:
				print "Error in database or database not compatible!!!"
				

			# we try to reach the file without telling the user, then the CD is probably in the drive
			if os.access(file, os.R_OK)==1:
				self.tempFile(file)
				continue

			else:
				pass

			while 1:
			    mb=self.pleaseInsertMessageBox(storagename)

        		    if mb.result() == QMessageBox.Yes:

                		self.takeSomeTime()

				try:
					self.tempFile(file)
					break
				except:
					print "wrong cd"

  			    else:
				notFoundDeps.append(dep)
				break


	    if notFoundDeps!=[]:
			testStatus=''
			for nd in notFoundDeps:
				if testStatus=='':
					testStatus=nd
				else:
					testStatus=testStatus+', '+nd


			text2=self.name + " needs the following to work correctly:<br><br><b>" + testStatus + """</b>
<br><br>You should probably install packages containing these before continuing."""
			self.warningMsg(text2)


    def pleaseInsertMessageBox(self, storagename):
	"""Handle the messagebox that asks the user to insert a specific media (CD)."""
	text="""Please insert <b>"""+storagename+"""</b> into '/mnt/cdrom' and press 'OK' when you are ready. Press 'Cancel' if you don't
	have that media.""" 
	
	#'/mnt/cdrom': change that so that it will read from the file which media it
	#should be inserted to OR BETTER check all /mnt/cdrom* to find the right CD

	mb = QMessageBox(NAME, text, QMessageBox.Information, QMessageBox.Yes | QMessageBox.Default , QMessageBox.Cancel, QMessageBox.NoButton, self)
	mb.setButtonText( QMessageBox.Yes, "OK" )
	mb.setButtonText( QMessageBox.Cancel, "Cancel" )
	#mb.show()
	mb.exec_loop()
	
	return mb

    def tempFile(self, file):
	"""Copy the file to a temporary directory and the temp file name to self.dep_satisfiers"""

    	tmpfilename=tempfile.mktemp('.rpm')
	shutil.copyfile(file, tmpfilename)
	self.tempfiles.append(tmpfilename)
	hdr, bull=self.getHeader(tmpfilename)
	self.dep_satisfiers.append((hdr, tmpfilename))


    def warningMsg(self,text2):
        """Handle the messagebox that warns the user when some problem has occured (missing dependencies etc.)"""
	text=text2+"""
<br><br>Press 'Continue' to install despite the problem or press 'Restart' to install another package.
Pressing 'Exit' will exit the whole wizard.
<br><br>What do you want to do?"""

	mb = QMessageBox("Warning", text, QMessageBox.Warning, QMessageBox.Yes | QMessageBox.Default, QMessageBox.No , QMessageBox.Cancel, self)
	
	mb.setButtonText( QMessageBox.Yes, "Continue" )
	mb.setButtonText( QMessageBox.No, "Restart" )
	mb.setButtonText( QMessageBox.Cancel, "Exit" )
	
	#mb.show()
	mb.exec_loop()

        if mb.result() == QMessageBox.Yes:
        	self.cont = 2
                self.takeSomeTime()

	elif mb.result() == QMessageBox.No:
        	self.cont=0

  	else:
		sys.exit(0)
	

    def startAtPage2(self, file):
    	"""Handle some things that have to be done when the wizard is started at the second page."""
	qfi=QFileInfo(file)
	fileAbs=qfi.absFilePath()
        self.pageOne.LineEdit1.setText(fileAbs)

	if self.setPage2()==0:
        	self.currentp=2
		self.setBackEnabled(self.pageTwo, 0)
        else:
        	self.currentp=1

    def setPage2(self, firstPage=1):
    	"""Set the info on the second page and decide whether the second page
	is allowed to be desplayed."""
    	self.critical_errorMsg=testFile(self.fileName(), self.fileName(1))

	if self.critical_errorMsg=='':
		self.currentp=2
		self.header, self.isSource=self.getHeader(self.fileName(1))
                self.getInfo()

                if firstPage==0:
                	self.pageTwo.TextLabel1.hide() # hide welcome text

                else:
                	pass
		self.setCaption(self.name + ' - ' + NAME)
               	self.showPage(self.pageTwo)
                self.backButton().setEnabled(1) 
		return 0
	else:
		self.mb2=QMessageBox.critical(self, NAME , '<b>Error:</b><br>'+self.critical_errorMsg, QMessageBox.Ok | QMessageBox.Default)
		return 1

    def callback(self, what, amount, total, h, param):
    	"""Is used as callback function during installation of RPM package."""

	if h!=None:
		(nam, file)=h


	if what == rpm.RPMCALLBACK_TRANS_START:
	    if amount == 6:
		self.pageThree.page3a.setText(self.tr('Preparing to install...'))

	if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
	    self.pageThree.changePage('b')
	    self.pageThree.ProgressBar33.setProgress(0)

	    self.rpmFD = os.open(file, os.O_RDONLY)
	    return self.rpmFD

	elif what == rpm.RPMCALLBACK_INST_PROGRESS:
	    if total:
		perc=long(amount)*100/total
		
		self.pageThree.ProgressBar33.setProgress(perc)

		self.pageThree.TextLabel2_3.setText("Installing: "+nam+"...")
		if perc==100:
			self.installOK=1

	elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
	    os.close(self.rpmFD)
	else:
	    pass
	qApp.processEvents()

    def testCallback(self, what, amount, total, h, param):
    	"""Used as callback function during testing of RPM package."""
	qApp.processEvents()
	
	if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
	    self.rpmFD = os.open(self.fileName(), os.O_RDONLY)
	    return self.rpmFD

	elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
	    os.close(self.rpmFD)

	else:
	    pass

