#*****************************************************
# RPM Wizard
# Copyright 2001-2002 Magnus Bjorklof <bjorklof@nic.fi>
# License: GNU General Public License
#*****************************************************


import commands
import sys
import getopt
import os, string
import rpm
import databasereader
from rpmwizdbcreator import *


from conf import *
from qt import *

def help_message():

    print NAME+" "+APPVERSION+"""

Usage: rpmwiz <rpm-file>  or rpmwiz <options>

Options:
  -h     --help              display this help message and exit
  -v     --version           display version and exit"""
    sys.exit(0)


def cltest():
        """Check if there are any arguments, if not print help message and exit"""

        filename=''


        try:
                options, xarguments = getopt.getopt(sys.argv[1:], 'hv', ['help', 'version'])

        except getopt.GetoptError:
                print "Error: unknown option\n"
                help_message()

        for a in options[:]:
                if a[0] == '-h' or a[0] == '--help':
                        help_message()

        for a in options[:]:
                if a[0] == '--version' or a[0] == '-v':
                        print NAME + ' ' + APPVERSION + u'\nCopyright (c) 2001-2002 Magnus Bjorklof '+EMAIL
                        sys.exit(0)


        return filename



def testFile(filename, filename1):
        """Test if the file exists and can be read.
        Also test if the file is a valid RPM file. Only the first error messsage
        that appears is displayed."""

        error_message=''

        qdir=QDir(filename)
        if qdir.exists()==1:
        	print "Error: No file specified"
		error_message="No file specified"
                return error_message
        else:
        	pass

        if os.access(filename, os.F_OK)==1:
                pass
        else:
                print "Error: No such file"
		error_message="No such file: "+ filename
                return error_message

        if os.access(filename, os.R_OK)==1:
                pass
        else:
                print "Error: File can't be read"
                error_message="File can't be read."
                return error_message

	

	testCmd = "rpm -qp --queryformat %{NAME} " + filename1
	q, q2=commands.getstatusoutput(testCmd)
	
	if q!=0:
            	print "Error: Not an RPM file"
            	error_message="The requested file is not an RPM file."
            	return error_message
	
        return error_message

def isKernel(name, isSource):
	"""Test if file is a kernel. This should be made more advanced..."""
	if name=='kernel' and isSource==0:
		return 1
	else:
		return 0

def getHeader(file):
	fd = os.open(file, os.O_RDONLY)
        h=rpm.headerFromPackage(fd)
	os.close(fd)
	return h

def depCheck(self, header, databases, details_dialog=0):
	"""Check for dependencies and return of missing dependencies etc. If it is run from page3 it will
	also check for files that can satisfy the missing dependencies."""

	db=rpm.opendb()
	ts=rpm.TransactionSet('/', db)
	ts.add(header, '', "u")


	depcheck = ts.depcheck()

	md=None
	if depcheck!=None:
		md, cf, ipcf, dss=ak74(depcheck, header)
	
	del ts
	del db

	list=[]
	stillUnsatisfied=[]

	if details_dialog==0 and md!=None and md!='':
		missingDeps=string.split(md, ',')
		list=databasereader.searchDepSatisfier(missingDeps, databases)
		
		if len(missingDeps)==len(list):
			print "All deps could be resolved"
			return None, None, None, list
		else:
			for item in missingDeps:
				for d_item in list:
					if string.strip(item)==string.strip(d_item[1]):
						break
				else:
					stillUnsatisfied.append(string.strip(item))


	missing=''
	if depcheck!=None:
		if stillUnsatisfied!=[]:
		    for m in stillUnsatisfied:
			if missing=='':
					missing=m
			else:
					missing=missing+', '+m

		    return missing, cf, ipcf, list

		else:
			return md, cf, ipcf, dss

	else:
		return None, None, None, []


def ak74(deps, header):
	"""Handles the list of tuples that is got from depcheck()"""
	missing_deps=''
	ip_missing_deps=None
	conflicts=None

	dep_satisfiers=[]

	for ((name, version, release),(reqname, reqversion), flags, suggest, sense) in deps:
                if sense == rpm.RPMDEP_SENSE_REQUIRES:
                    if suggest: #this is unneccessery with my current design
                        (hedr, file) = suggest
			try:
				dep_satisfiers.index((suggest, reqname, '', 'file' ))
			except:
				print "suggest", suggest

				for ((h, filen), rn, dkds,fsdffs) in dep_satisfiers:
					if filen==file or rn==reqname:
						break
				else:
					dep_satisfiers.append((suggest, reqname, '', 'file' ))


                    else:
			if name==header[rpm.RPMTAG_NAME]:
				reqname2=makeRequireString(reqname, reqversion, flags)

				print name, "missing dependency", reqname2
				if missing_deps=='':
					missing_deps=reqname2
				else:
					missing_deps=missing_deps+', '+reqname2
			else:
				print "already installed package ", name, " will be unhappy if we install"
				if ip_missing_deps==None:
					ip_missing_deps=name
				else:
					ip_missing_deps=ip_missing_deps+', '+name
                elif sense == rpm.RPMDEP_SENSE_CONFLICTS:
		    print reqname, " conflicts with ", name
		    if conflicts==None:

		    	conflicts=reqname


	return (missing_deps, conflicts, ip_missing_deps, dep_satisfiers)


def userTest():
        """Tests if the user has root privileges. If not it tries to find kdesu."""
        user=commands.getoutput('whoami')
        if user=='root':
                isRoot=1
        else:
		KDIR=os.getenv('KDEDIR')
		if KDIR!=None and KDIR!='':
			pass
		else:
			KDIR='/usr/'
		
		try:
			os.execv(KDIR+'/bin/kdesu', [sys.argv[0], '-c' ,'rpmwiz']+sys.argv[1:])
		except OSError:
			try:
				os.execv('/opt/kde2/bin/kdesu', [sys.argv[0], '-c' ,'rpmwiz']+sys.argv[1:])
			except OSError:
				pass
		isRoot=0

        return isRoot
